<?php

/*
cgi-bin/calendar.php
Copyright 2005 Lorne McIntosh
www.lorneswork.com
*/

require("cgi-bin/db_connect.php");
require("cgi-bin/csscolor.php");

// chop a string into a smaller string
function str_chop($string, $length = 20, $append = '...')
{	
	// get some measurements
	$len_string = strlen($string);
	$len_append = strlen($append);
	
	// if the string is longer than the maximum length, we need to chop it
	if ($len_string > $length)
	{
		// chop the end off
		$string = substr($string, 0, $length - $len_append) . $append;
	}

	return $string;
}

//displays date in the users timezone
function tzDate($date_format, $timestamp)
{ 
	//tz offset is stored in minutes from GMT
	$timestamp += (60 * $_SESSION['tz']); 
    return gmdate($date_format, $timestamp); 
	//using 'date' will adjust 1 hour for daylight saving too, if the SERVER is currently in daylight saving time
	//(this makes DST mandatory for all users, and should really be optional - switch between 'gmdate' and 'date' on a user by user basis?)
}

function dispDate()
{
	global $logged_in;
	if($logged_in)
	{
		//local user time
		echo tzDate("l, F jS Y - g:ia", time());
	}
	else
	{
		//local server time
		echo date("l, F jS Y - g:ia (T)");
	}
}

class WebCalendar
{
	var $mini;
	var $offset;
	
	var $now;
	
	var $year;
	var $month;
	
	var $startTime;

	// Constructor:
	// $mini - removes events, adds title link, and uses different css style
	// $simple - doesn't display any events from database, no links
	// $offset - offsets the month being displayed from the one in the URL (use negative numbers for previous months)
	function WebCalendar($mini, $simple, $offset)
	{
		//calendar options
		$this->mini = $mini;
		$this->simple = $simple;
		$this->offset = $offset;
		
		// get current date for user
		$this->now['year'] = tzDate("Y", time());
		$this->now['mon'] = tzDate("n", time());
		$this->now['mday'] = tzDate("j", time());
		
		// get URL parameters or defaults
		if(!isset($_GET['year']))
		{
			$this->year = $this->now["year"]; // if it's null, use default value
		}	
		else
		{
			$this->year = $_GET['year']; // use passed value
		}
		
		if(!isset($_GET['month']))
		{
			$this->month = $this->now["mon"] + $this->offset; // if it's null, use default value
		}
		else
		{
			$this->month = $_GET['month'] + $this->offset; // use passed value
		}
		
		// if we are at 'month 13'
		if($this->month > 12)
		{
			$this->month = 1;
			$this->year++;
		}
		
		// if we are at 'month 0'
		if($this->month < 1)
		{
			$this->month = 12;
			$this->year--;
		}
		
		//used for calculating some stuff
		$this->startTime = strtotime($this->year . "-" . $this->month . "-01 00:00");
	}
	
	function dispMonthTitle()
	{
		if ($this->offset != 0)
		{
			//link to it
			echo "<div align=\"center\" class=\"monthTitle\"><a href=\"" . $_SERVER['PHP_SELF'] . "?year=" . $this->year . "&month=" . $this->month . "\">" . gmdate("F Y", $this->startTime) . "</a></div>\n";
		}
		else
		{
			//don't link to it - we're already there
			echo "<div align=\"center\" class=\"monthTitle\">" . gmdate("F Y", $this->startTime) . "</div>\n";
		}
	}
	
	function dispMonth()
	{
		echo "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\">\n";
		$this->dispDayNames();
		$this->dispDayCells();
		echo "</table>\n";
	}
	
	function dayName($day, $format)
	{
		$names = array('sun','mon','tue','wed','thu','fri','sat');
		return strftime($format, strtotime($names[$day]));
	}
	
	function dispDayNames()
	{
		echo "<tr>\n";
		for($i = 0; $i < 7; $i++)
		{
			if(!$this->mini) {echo "\t<th width=\"14%\">" . $this->dayName($i, "%A") . "</th>\n";}
			if($this->mini) {echo "\t<th width=\"14%\">" . $this->dayName($i, "%a") . "</th>\n";}
		}        
		echo "</tr>\n";
	}

	function dispDayCells()
	{
		$i = 0; // cell counter

		echo "<tr>\n";
		
		// display any empty cells at start of month
		for ($d = 0; $d < gmdate('w', $this->startTime); $d++)
		{
			echo "\t<td class='notInMonth'>&nbsp;</td>\n";
			$i++;
		}
		
		// display the days in the month, start a new row after every week
		for($d = 0; $d < gmdate('t', $this->startTime); $d++)
		{
			$date = $d + 1;

			//this calendar isn't 'simple' so link each cell to the corresponding date, and show events etc.
			if (!$this->simple)
			{
				// is this one representing today?
				if	(($this->year == $this->now["year"]) &&
					($this->month == $this->now["mon"]) &&
					($date == $this->now["mday"]))
				{
					echo "\t<td class=\"now\" onClick=\"popup('viewevent','day.php?year=" . $this->year . "&month=" . $this->month . "&day=" . $date . "',604,528)\" " .
						"style='cursor: pointer; cursor: hand;' onmouseover=\"this.className='highlight'\" onmouseout=\"this.className='now'\">";
				}
				else
				{
					echo "\t<td onClick=\"popup('viewevent','day.php?year=" . $this->year . "&month=" . $this->month . "&day=" . $date . "',604,528)\" " .
						"style='cursor: pointer; cursor: hand;' onmouseover=\"this.className='highlight'\" onmouseout=\"this.className='td'\">";
				}
			
				//show cell contents
				$this->dispCellContents($date);
			}
		
			//this calendar is 'simple' so just show the date
			else
			{
				// is this one representing today?
				if	(($this->year == $this->now["year"]) &&
					($this->month == $this->now["mon"]) &&
					($date == $this->now["mday"]))
				{
					echo "\t<td class=\"now\">";
				}
				else
				{
					echo "\t<td>";
				}
				echo "<div class=\"dateWithoutEvent\">$date</div>\n";
			}
			
			//close off cell
			echo "</td>\n";
			
			$i++;
			//close off row
			if (($i % 7) == 0)
			{
				echo "</tr>\n<tr>\n";
			}
		}
		
		// display any empty cells at end of month
		$cellsRemaining = 7 - ($i % 7);
		if ($cellsRemaining < 7)  
		{
			for ($c = 0; $c < $cellsRemaining; $c++)
			{
				echo "\t<td class='notInMonth'>&nbsp;</td>\n";
			}
			echo "</tr>\n";
		}    
	}
	
	// loads the contents of a given cell
	function dispCellContents($date)
	{
		//get our database objects
		global $db_object;
		global $eventsTable;
		global $attendanceTable;

		//mysql will do the timezone conversion from GMT for us - we just provide the offset in minutes
		$sql = "SELECT id, owner, title, description, location, DATE_ADD(startDT, INTERVAL '".$_SESSION['tz']."' MINUTE) AS startDT, DATE_ADD(endDT, INTERVAL '".$_SESSION['tz']."' MINUTE) AS endDT, color
			FROM ".$eventsTable."
			RIGHT JOIN ".$attendanceTable." ON id = eid
			WHERE uid = '".$_SESSION['id']."'
			AND (DATE_FORMAT(DATE_ADD(startDT, INTERVAL '".$_SESSION['tz']."' MINUTE), '%Y-%c-%e') = '".$this->year."-".$this->month."-".$date."'
			OR DATE_FORMAT(DATE_ADD(endDT, INTERVAL '".$_SESSION['tz']."' MINUTE), '%Y-%c-%e') = '".$this->year."-".$this->month."-".$date."')
			ORDER BY startDT
			";
		$query = $db_object->query($sql);
		if (DB::isError($query))
		{
			die($query->getMessage());
		}

		if($query->numRows() == 0)
		{
			// no event on this day, so just show regular date
			echo "<div class=\"dateWithoutEvent\">$date</div>\n";
		}
		else
		{
			echo "<div class=\"dateWithEvent\">$date</div>\n";
			
			// if this is a full-size calendar
			if(!$this->mini)
			{
				echo "\t<hr>\n";
				echo "\t<div class=\"events\">\n";
				
				// display each event, one at a time
				while ($row = $query->fetchRow())
				{
					if($row['startDT']) {$row['startDT'] = date("g:ia", strtotime($row['startDT']));}
					if($row['endDT']) {$row['endDT'] = date("g:ia", strtotime($row['endDT']));}
					
					$base = new CSS_Color($row['color']);
					
					// alternate text
					echo "\t<a class=\"events\" title=\"";
					if ($row['startDT'])
					{
						echo $row['startDT'] . " ";
						if ($row['endDT'])
						{
							echo "- " . $row['endDT'] . ": ";
						}
					}
					echo $row['title'];
					if($row['location'])
					{
						echo " (" . $row['location'] . ")";
					}
					
					// normal text
					echo "\"><font color=\"#" . $base->bg['-3'] . "\">" . $row['startDT'] . " <b>" . str_chop($row['title'], 8) . "</b></font></a><br>\n";
				}
				
				echo "\t</div>\n";
			}
		}
	}

	function dispControls()
	{
		echo "<form method=\"get\" action=\"" . $_SERVER['PHP_SELF'] . "\">\n";
        echo "<select name=\"month\">\n";
		
		$monthName = array('January','February','March','April','May','June','July','August','September','October','November','December');
		for ($i = 1; $i <= 12; $i++)
		{
			if($i == $this->month)
			{
				echo "\t<option selected value=\"" . $i . "\">" . $monthName[$i-1] . "</option>\n";
			}
			else
			{
				echo "\t<option value=\"" . $i . "\">" . $monthName[$i-1] . "</option>\n";
			}
		}        
        echo "</select>\n";
		
		echo "<select name=\"year\">\n";
		
		for ($i = $this->year - 4; $i <= $this->year + 4; $i++)
		{
			if($i == $this->year)
			{
				echo "\t<option selected>$i</option>\n";
			}
			else
			{
				echo "\t<option>$i</option>\n";
			}
		}
        
		echo "</select>\n";
		echo "<INPUT type=\"submit\" value=\"Go\">\n";
      	echo "</form>";
	}
}

?>
